/* code taken from linux-2.4.17/drivers/char/nvram.c 
 * 
 * nvram.c
 */

/* Read a byte from a port.  */

#ifdef HAVENVRAM
#WARNING nvram.c included more then once
#else
#define HAVENVRAM 1

#ifndef _SYS_IO_H
static inline unsigned char
volatile inb (unsigned short port)
{
  volatile unsigned char value;
  
  asm volatile ("inb	%w1, %0" : "=a" (value) : "Nd" (port));
//  if (debug) grub_printf ("read %x from nvram addr %x\n",value,port);
  return value;
}

/* Write a byte to a port.  */
static inline void
outb ( volatile unsigned char value, unsigned short port)
{
  asm volatile ("outb	%b0, %w1" : : "a" (value), "Nd" (port));
//  if (debug) grub_printf ("wrote %x to nvram addr %x\n",value,port);
}
#endif /* sys/io.h */
static inline unsigned char nvram_read( int i )
{
	outb (i+14,0x70);
	return inb (0x71);
}

static inline void nvram_write( unsigned char c, int i )
{
	outb (i+14,0x70);
	outb (c,0x71);
}

static inline int nvram_check_checksum()
{
        int i;
        unsigned short sum = 0;

        for( i = 2; i <= 31; ++i )
                sum += nvram_read( i );
        return( (sum & 0xffff) ==
                        ((nvram_read(32) << 8) |
                         nvram_read(32 +1)) );
}

static inline int nvram_check_and_print_checksum()
{
        int i,j;
        unsigned short sum = 0;
	grub_printf("NVRAM contents:\n");
        for( i = 2; i <= 31; ++i ){
            	j= nvram_read( i );
		sum += j;
//		grub_printf("%x:%x %x  ",i,j,sum);
		};
	grub_printf("\n==== calculated chechsum=%x , stored value=%x\n", 
		(sum & 0xffff), ((nvram_read(32) << 8) | nvram_read(33)) );
	for ( i=1;i <=128 ; i++ ){
		j= nvram_read (i);
		if (!((i+15)%16)) grub_printf("0x%X [ ",i);
		if (!((i+7)%16)) grub_printf("|");
		if (!((i+3)%8)) grub_printf(":");
		if (!((i+1)%4)) grub_printf(",");
		if (j<0x10) grub_printf("0");
		grub_printf("%x",j);
		if (!(i%16)) grub_printf("]\n");
		};
	return( (sum & 0xffff) ==
                        ((nvram_read(32) << 8) |
                         nvram_read(33)) );
			 
}

static inline void nvram_set_checksum()
{
        int i;
        unsigned short sum = 0;
        for( i = 2; i <= 31; ++i )
                sum += nvram_read( i );
        nvram_write( sum >> 8, 32 );
        nvram_write( sum & 0xff, 32 +1 );
}
#endif /* have nvram */

